export declare const aStarAssocTraversal: <T>(getNextStates: (n: T) => [T, number][], estimateRemainingCost: (n: T) => number, initial: T) => Generator<[number, T[]]>;
export declare const aStarTraversal: <T>(getNextStates: (n: T) => T[], getCost: (a: T, b: T) => number, estimateRemainingCost: (n: T) => number, initial: T) => Generator<[number, T[]]>;
/**
 * Performs a best-first search
 * using the A* search algorithm, starting with the state @initial@, generating
 * neighboring states and their associated costs with @next@, and an estimate of
 * the remaining cost with @remaining@. This returns a path to a state for which
 * @found@ returns 'True'. If @remaining@ is strictly a lower bound on the
 * remaining cost to reach a solved state, then the returned path is the
 * shortest path. Returns 'Nothing' if no path to a solved state is possible.
 *
 * @public
 * @param getNextStates - Function to generate list of neighboring states with associated transition costs given the current state
 * @param estimateRemainingCost - Estimate on remaining cost given a state
 * @param determineIfFound - Predicate to determine if solution found. `aStar` returns the shortest path to the first state for which this predicate returns `true`
 * @param start - starting state
 * @returns [Total cost, list of steps] for the first path found which satisfies the given predicate
 */
export declare const aStarAssoc: <T>(getNextStates: (n: T) => [T, number][], estimateRemainingCost: (n: T) => number, determineIfFound: (a: T) => boolean, start: T) => [number, T[], T[]] | undefined;
/**
 * Performs a best-first search
 * using the A* search algorithm, starting with the state @initial@, generating
 * neighboring states with `next`, their cost with @cost@, and an estimate of
 * the remaining cost with `remaining`. This returns a path to a state for which
 * `found` returns `true`. If `remaining` is strictly a lower bound on the
 * remaining cost to reach a solved state, then the returned path is the
 * shortest path. Returns `undefined` if no path to a solved state is possible.
 *
 * @public
 * @param getNextStates - Function to generate list of neighboring states given the current state
 * @param getCost - Function to generate transition costs between neighboring states
 * @param estimateRemainingCost - Estimate on remaining cost given a state
 * @param determineIfFound - Predicate to determine if solution found. `aStar` returns the shortest path to the first state for which this predicate returns `true`
 * @param initial - Initial state
 * @returns - [Total cost, list of steps] for the first path found which satisfies the given predicate
 */
export declare const aStar: <T>(getNextStates: (n: T) => T[], getCost: (a: T, b: T) => number, estimateRemainingCost: (n: T) => number, determineIfFound: (a: T) => boolean, initial: T) => [number, T[], T[]] | undefined;
