'use strict';

var ramda = require('ramda');

/** @internal */
const createPath = (prevMap, final) => {
    const path = [final];
    let prev = prevMap.get(ramda.toString(final));
    while (prev != null) {
        path.unshift(prev);
        prev = prevMap.get(ramda.toString(prev));
    }
    return path;
};

const top = 0;
/* eslint-disable no-bitwise */
const parent = (i) => ((i + 1) >>> 1) - 1;
// double - 1
const left = (i) => (i << 1) + 1;
// double + 2
const right = (i) => (i + 1) << 1;
/* eslint-enable no-bitwise */
const priorityQueue = (comparator) => {
    const heap = [];
    const size = () => heap.length;
    const isEmpty = () => size() === 0;
    const peek = () => heap[top];
    const greater = (i, j) => comparator(heap[i], heap[j]);
    const swap = (i, j) => {
        const a = heap[i];
        const b = heap[j];
        heap[j] = a;
        heap[i] = b;
    };
    const siftUp = () => {
        let node = size() - 1;
        while (node > top && greater(node, parent(node))) {
            swap(node, parent(node));
            node = parent(node);
        }
    };
    const siftDown = () => {
        let node = top;
        while ((left(node) < size() && greater(left(node), node)) || (right(node) < size() && greater(right(node), node))) {
            const maxChild = right(node) < size() && greater(right(node), left(node)) ? right(node) : left(node);
            swap(node, maxChild);
            node = maxChild;
        }
    };
    const replace = (value) => {
        const replacedValue = peek();
        heap[top] = value;
        siftDown();
        return replacedValue;
    };
    const push = (value) => {
        heap.push(value);
        siftUp();
    };
    const pop = () => {
        const poppedValue = peek();
        const bottom = size() - 1;
        if (bottom > top) {
            swap(top, bottom);
        }
        heap.pop();
        siftDown();
        return poppedValue;
    };
    return { isEmpty, peek, pop, push, replace, size };
};

exports.createPath = createPath;
exports.priorityQueue = priorityQueue;
//# sourceMappingURL=priorityQueue.cjs.map
